#include "modem.h"

#include <modem/nrf_modem_lib.h>
#include <zephyr/logging/log.h>

K_SEM_DEFINE(lte_connected, 0, 1);
K_SEM_DEFINE(modem_shutdown_sem, 0, 1);

LOG_MODULE_REGISTER(Modem, LOG_LEVEL_DBG);

void lte_handler(const struct lte_lc_evt* const evt) {
    switch (evt->type) {
        case LTE_LC_EVT_NW_REG_STATUS:
            if ((evt->nw_reg_status != LTE_LC_NW_REG_REGISTERED_HOME) &&
                (evt->nw_reg_status != LTE_LC_NW_REG_REGISTERED_ROAMING)) {
                break;
            }
            LOG_INF("Network registration status: %s", evt->nw_reg_status == LTE_LC_NW_REG_REGISTERED_HOME
                                                           ? "Connected - home network"
                                                           : "Connected - roaming");
            k_sem_give(&lte_connected);
            break;
        case LTE_LC_EVT_RRC_UPDATE:
            LOG_INF("RRC mode: %s", evt->rrc_mode == LTE_LC_RRC_MODE_CONNECTED ? "Connected" : "Idle");
            break;

        case LTE_LC_EVT_CELL_UPDATE:
            LOG_INF("LTE cell changed: Cell ID: %d, Tracking area: %d\n", evt->cell.id, evt->cell.tac);
            break;

        default:
            break;
    }
}

int modem_configure(void) {
    int err;

    LOG_INF("Initializing modem library");

    err = nrf_modem_lib_init();
    if (err) {
        LOG_ERR("Failed to initialize the modem library, error: %d", err);
        return err;
    }

    LOG_INF("Connecting to LTE network");
    err = lte_lc_connect_async(lte_handler);
    if (err) {
        LOG_ERR("Error in lte_lc_connect_async, error: %d", err);
        return err;
    }

    k_sem_take(&lte_connected, K_FOREVER);
    LOG_INF("Connected to LTE network");

    struct lte_lc_conn_eval_params params;

    int res = lte_lc_conn_eval_params_get(&params);
    if (res < 0) {
        LOG_ERR("Failed to get connection evaluation parameters, error: %d", res);
        return res;
    }
    LOG_INF("RRC State: %d", params.rrc_state);
    LOG_INF("Energy Estimate: %d", params.energy_estimate);
    LOG_INF("TAU Triggered: %d", params.tau_trig);
    LOG_INF("CE Level: %d", params.ce_level);
    LOG_INF("EARFCN: %d", params.earfcn);
    LOG_INF("DL Pathloss: %d", params.dl_pathloss);
    LOG_INF("RSRP: %d", params.rsrp);
    LOG_INF("RSRQ: %d", params.rsrq);
    LOG_INF("TX Repetitions: %d", params.tx_rep);
    LOG_INF("RX Repetitions: %d", params.rx_rep);
    LOG_INF("PHY CID: %d", params.phy_cid);
    LOG_INF("Band: %d", params.band);
    LOG_INF("SNR: %d", params.snr);
    LOG_INF("TX Power: %d", params.tx_power);
    LOG_INF("MCC: %d", params.mcc);
    LOG_INF("MNC: %d", params.mnc);
    LOG_INF("Cell ID: %u", params.cell_id);

    return 0;
}