#include "tcp.h"

#include <zephyr/logging/log.h>

#define SERVER_HOSTNAME "tcpbin.com"
#define SERVER_PORT "4242"

// #define SERVER_HOSTNAME "udp-echo.nordicsemi.academy"
// #define SERVER_PORT "2444"

LOG_MODULE_REGISTER(LTE, LOG_LEVEL_DBG);

#define MAX_RETRIES 3

int server_resolve(void) {
    int err;
    struct addrinfo* result;
    struct addrinfo hints = {.ai_family = AF_INET, .ai_socktype = SOCK_STREAM};

    LOG_INF("Resolving server name: %s, port: %s", SERVER_HOSTNAME, SERVER_PORT);

    err = getaddrinfo(SERVER_HOSTNAME, SERVER_PORT, &hints, &result);
    if (err != 0) {
        LOG_ERR("ERROR: getaddrinfo failed %d: %s\n", err, gai_strerror(err));
        return -EIO;
    }

    if (result == NULL) {
        LOG_ERR("ERROR: Address not found\n");
        return -ENOENT;
    }

    struct sockaddr_in* server4 = ((struct sockaddr_in*)&server);
    server4->sin_addr.s_addr = ((struct sockaddr_in*)result->ai_addr)->sin_addr.s_addr;
    server4->sin_family = AF_INET;
    server4->sin_port = ((struct sockaddr_in*)result->ai_addr)->sin_port;

    char ipv4_addr[NET_IPV4_ADDR_LEN];
    inet_ntop(AF_INET, &server4->sin_addr.s_addr, ipv4_addr, sizeof(ipv4_addr));
    LOG_INF("IPv4 Address found %s", ipv4_addr);

    freeaddrinfo(result);

    return 0;
}

int server_connect(void) {
    int err;
    int retries = 0;

    while (retries < MAX_RETRIES) {
        LOG_INF("Creating socket");
        sock = socket(AF_INET, SOCK_STREAM,
                      /* IPPROTO_IP */ IPPROTO_TCP);
        if (sock < 0) {
            LOG_ERR("Failed to create socket: %d.\n", errno);
            return -errno;
        }

        LOG_INF("Connecting to server");
        err = connect(sock, (struct sockaddr*)&server, sizeof(struct sockaddr_in));
        if (err < 0) {
            LOG_ERR("Connect failed : %d\n", errno);
            close(sock);
            retries++;
            k_sleep(K_SECONDS(1));
        } else {
            LOG_INF("Successfully connected to server");
            return 0;
        }
    }

    LOG_ERR("Failed to connect to server after %d retries", MAX_RETRIES);
    return -ETIMEDOUT;
}
