import os

from pydantic import ValidationError
from pydantic_settings import BaseSettings, SettingsConfigDict

ENV_FILE = os.getenv("ENV_FILE", ".env")


class LoggingSettings(BaseSettings):
    """Settings related with the debug logging"""

    model_config = SettingsConfigDict(env_file=ENV_FILE, env_prefix="LOG_")

    level: str = "INFO"
    serialize: bool = False


class SerialSettings(BaseSettings):
    """Settings related with the serial port"""

    model_config = SettingsConfigDict(env_file=ENV_FILE, env_prefix="SERIAL_")

    port: str = "/dev/ttyACM1"
    baudrate: int = 1000000


try:
    logging_settings = LoggingSettings()
    serial_settings = SerialSettings()
except ValidationError as error:
    print(f"Please check your settings ({error})")
    exit(-1)
